@@ -45,6 +45,7 @@ module Yesod.Form.Functions
     , fieldSettingsLabel
     , parseHelper
     , parseHelperGen
+    , convertField
     ) where
 
 import Yesod.Form.Types
@@ -222,6 +223,7 @@ postHelper form env = do
     ((res, xml), enctype) <- runFormGeneric (form token) m langs env
     let res' =
             case (res, env) of
+                (_, Nothing) -> FormMissing
                 (FormSuccess{}, Just (params, _))
                     | not (Map.lookup tokenKey params === reqToken req) ->
                         FormFailure [renderMessage m langs MsgCsrfWarning]
@@ -253,8 +255,8 @@ postEnv = do
             return $ Just (p', Map.unionsWith (++) $ map (\(k, v) -> Map.singleton k [v]) f)
 
 runFormPostNoToken :: MonadHandler m
-                   => (Html -> MForm m (FormResult a, xml))
-                   -> m ((FormResult a, xml), Enctype)
+                   => (Html -> MForm m a)
+                   -> m (a, Enctype)
 runFormPostNoToken form = do
     langs <- languages
     m <- getYesod
@@ -272,7 +274,7 @@ runFormGet form = do
                 Just _ -> Just (Map.unionsWith (++) $ map (\(x, y) -> Map.singleton x [y]) gets, Map.empty)
     getHelper form env
 
-{- FIXME: generateFormGet' "Will be renamed to generateFormGet in next verison of Yesod" -}
+{- FIXME: generateFormGet' "Will be renamed to generateFormGet in next version of Yesod" -}
 -- |
 --
 -- Since 1.3.11
@@ -282,7 +284,7 @@ generateFormGet'
     -> m (xml, Enctype)
 generateFormGet' form = first snd `liftM` getHelper form Nothing
 
-{-# DEPRECATED generateFormGet "Will require RenderMessage in next verison of Yesod" #-}
+{-# DEPRECATED generateFormGet "Will require RenderMessage in next version of Yesod" #-}
 generateFormGet :: MonadHandler m
                 => (Html -> MForm m a)
                 -> m (a, Enctype)
@@ -526,3 +528,29 @@ parseHelperGen :: (Monad m, RenderMessage site msg)
 parseHelperGen _ [] _ = return $ Right Nothing
 parseHelperGen _ ("":_) _ = return $ Right Nothing
 parseHelperGen f (x:_) _ = return $ either (Left . SomeMessage) (Right . Just) $ f x
+
+-- | Since a 'Field' cannot be a 'Functor', it is not obvious how to "reuse" a Field
+-- on a @newtype@ or otherwise equivalent type. This function allows you to convert
+-- a @Field m a@ to a @Field m b@ assuming you provide a bidireccional
+-- convertion among the two, through the first two functions.
+--
+-- A simple example:
+--
+-- > import Data.Monoid
+-- > sumField :: (Functor m, Monad m, RenderMessage (HandlerSite m) FormMessage) => Field m (Sum Int)
+-- > sumField = convertField Sum getSum intField
+--
+-- Another example, not using a newtype, but instead creating a Lazy Text field:
+--
+-- > import qualified Data.Text.Lazy as TL
+-- > TextField :: (Functor m, Monad m, RenderMessage (HandlerSite m) FormMessage) => Field m TL.Text
+-- > lazyTextField = convertField TL.fromStrict TL.toStrict textField
+--
+-- Since 1.3.16
+convertField :: (Functor m)
+             => (a -> b) -> (b -> a)
+             -> Field m a -> Field m b
+convertField to from (Field fParse fView fEnctype) = let
+  fParse' ts = fmap (fmap (fmap to)) . fParse ts
+  fView' ti tn at ei = fView ti tn at (fmap from ei)
+  in Field fParse' fView' fEnctype
